package de.hdm_stuttgart.mi.sd1.navigation;

/**
 * A route from one location to another will be divided
 * into segments:
 * 
 * Segment 1: Take the main road till reaching M2
 * Segment 2: Continue till reaching ...
 * ...
 * 
 * Each leg is being described by its length and speed limit.
 *
 */
public class Segment {
   
   /**
    * Indication a segment does not have any speed limit.
    */
   static public final int NO_SPEED_LIMIT = 0;

   /**
    * The current leg's distance in km.
    */
   public final double distance;
   
   /**
    * The current leg's speed limit in km / h. A value of 0 (zero) indicates
    * the absence of any speed limit, see variable {@link #NO_SPEED_LIMIT}
    */
   public final int speedLimit;
   
   /**
    * Creating an (immutable) leg having a distance and possibly a
    * speed limit.
    * 
    * @param distance {@link #distance}
    * @param speedLimit {@link #speedLimit}
    */
   public Segment(final double distance, final int speedLimit) {
      this.distance = distance;
      this.speedLimit = speedLimit;
   }
   
   /**
    * Minimal time required when consequently traveling with the minimum of
    * official speed limits and the driver's personal maximum speed. If a
    * leg does not have any speed limit the value of personalSpeedLimit will
    * be used for calculation instead.
    * 
    * @param route The array of segments composing a route.
    * 
    * @param personalSpeedLimit The drivers personal speed limit whether or 
    *        not official limits apply.
    * 
    * @return The minimal duration in (rounded) minutes with respect to all
    *         speed limits. Must be a positive (non-zero) value.
    */
   static public long duration(Segment[] route, int personalSpeedLimit) {
      double duration = 0; // Time in hours
      for (final Segment s : route) {
         if (s.speedLimit == 0 ||                   // No Official speed limit or above
               personalSpeedLimit < s.speedLimit) { // driver's personal maximum
            duration += s.distance / personalSpeedLimit;
         } else {
            duration += s.distance / s.speedLimit;
         }
      }
      return Math.round(60 * duration);
   }
}
